from datetime import datetime

class Script(object):
    
    contents = ''
    filename = 'kickstarter.nc'
    url = 'http://publicwiki.deltares.nl/display/OET/netCDF%20kickstarter'

    def __init__(self, filename=None, url=None):
        if not filename == None:
            self.filename = filename
        if not url == None:
            self.url = url

    def get_contents(self):
        return self.contents

    def add_header(self):
        
        self.add_comment('%s script generated by netCDF kickstarter' % self.__class__.__name__)
        self.add_comment('at %s' % datetime.strftime(datetime.utcnow(), '%Y-%m-%dT%H:%MZ'))
        self.add_comment(self.url)
        self.add_empty()

    def add_footer(self):
        pass
        
    def add_comment(self, comment):
        self._add_line('# %s' % comment)

    def add_cell(self, comment):
        self._add_line('## %s' % comment)

    def add_dimension(self, name, length):
        pass

    def add_variable(self, name, dtype, dimensions):
        pass

    def add_attribute(self, name, value, variable=None):
        if variable == None:
            self.add_attribute_global(name, value)
        else:
            self.add_attribute_var(name, value, variable)

    def add_attribute_global(self, name, value):
        pass

    def add_attribute_var(self, name, value, variable):
        pass

    def add_empty(self):
        self._add_line()

    def _add_line(self, line=''):
        self.contents = self.contents + line + '\n'

    def _istext(self, value):
        if type(value) is str or type(value) is unicode:
            return True
        else:
            return False

class Python(Script):

    def add_header(self):
        super(Python, self).add_header()

        self._add_line('import netCDF4')
        self._add_line()
        self._add_line('nc = netCDF4.Dataset(\'%s\',\'w\')' % self.filename)
        self._add_line()

    def add_footer(self):
        super(Python, self).add_footer()

        self._add_line('nc.close()')

    def add_dimension(self, name, length):
        self._add_line('nc.createDimension(\'%s\', %d)' % (name, length))

    def add_variable(self, name, dtype, dimensions):
        self._add_line('nc.createVariable(\'%s\', \'%s\', %s)' % (name, str(dtype), repr(dimensions)))

    def add_attribute_global(self, name, value):
        if self._istext(value):
            self._add_line('nc.%s = \'%s\'' % (name, value))
        else:
            self._add_line('nc.%s = %s' % (name, value))

    def add_attribute_var(self, name, value, variable):
        if self._istext(value):
            self._add_line('nc.variables[\'%s\'].%s = \'%s\'' % (variable, name, value))
        else:
            self._add_line('nc.variables[\'%s\'].%s = %s' % (variable, name, value))
              
class Matlab(Script):

    def add_header(self):
        super(Matlab, self).add_header()

        self._add_line('nc_create_empty(\'%s\');' % self.filename)
        self._add_line()

    def add_footer(self):
        super(Matlab, self).add_footer()

    def add_comment(self, comment):
        self._add_line('%% %s' % comment)

    def add_cell(self, comment):
        self._add_line('%%%% %s' % comment)

    def add_dimension(self, name, length):
        self._add_line('nc_adddim(\'%s\', \'%s\', %d);' % (self.filename, name, length))

    def add_variable(self, name, dtype, dimensions):
        self._add_line('var = struct();')
        self._add_line('var.Name = \'%s\';' % name)
        self._add_line('var.Datatype = \'%s\';' % self._convert_dtype(dtype))
        self._add_line('var.Dimension = {%s};' % ','.join(['\'%s\'' % d for d in dimensions]))
        self._add_line('nc_addvar(\'%s\', var);' % self.filename)

    def add_attribute_global(self, name, value):
        if self._istext(value):
            self._add_line('nc_attput(\'%s\', nc_global, \'%s\', \'%s\');' % (self.filename, name, value))
        else:
            self._add_line('nc_attput(\'%s\', nc_global, \'%s\', %s);' % (self.filename, name, value))

    def add_attribute_var(self, name, value, variable):
        if self._istext(value):
            self._add_line('nc_attput(\'%s\', \'%s\', \'%s\', \'%s\');' % (self.filename, variable, name, value))
        else:
            self._add_line('nc_attput(\'%s\', \'%s\', \'%s\', %s);' % (self.filename, variable, name, value))

    def _convert_dtype(self, dtype):
        if dtype.startswith('float'):
            return 'double'
        else:
            return dtype

class Matlab(Script):

    def add_header(self):
        super(Matlab, self).add_header()

        self._add_line('nc_create_empty(\'%s\');' % self.filename)
        self._add_line()

    def add_footer(self):
        super(Matlab, self).add_footer()

    def add_comment(self, comment):
        self._add_line('%% %s' % comment)

    def add_dimension(self, name, length):
        self._add_line('nc_adddim(\'%s\', \'%s\', %d);' % (self.filename, name, length))

    def add_variable(self, name, dtype, dimensions):
        self._add_line('var = struct();')
        self._add_line('var.Name = \'%s\';' % name)
        self._add_line('var.Datatype = \'%s\';' % self._convert_dtype(dtype))
        self._add_line('var.Dimension = {%s};' % ','.join(['\'%s\'' % d for d in dimensions]))
        self._add_line('nc_addvar(\'%s\', var);' % self.filename)

    def add_attribute_global(self, name, value):
        if self._istext(value):
            self._add_line('nc_attput(\'%s\', nc_global, \'%s\', \'%s\');' % (self.filename, name, value))
        else:
            self._add_line('nc_attput(\'%s\', nc_global, \'%s\', %s);' % (self.filename, name, value))

    def add_attribute_var(self, name, value, variable):
        if self._istext(value):
            self._add_line('nc_attput(\'%s\', \'%s\', \'%s\', \'%s\');' % (self.filename, variable, name, value))
        else:
            self._add_line('nc_attput(\'%s\', \'%s\', \'%s\', %s);' % (self.filename, variable, name, value))

    def _convert_dtype(self, dtype):
        if dtype.startswith('float'):
            return 'double'
        else:
            return dtype
 
class MatlabNew(Script):

    def add_header(self):
        super(MatlabNew, self).add_header()

        self._add_line("ncfile = '%s';"%self.filename)
        self.add_empty()
        self._add_line('dims = [];')
        self._add_line('vars = [];')
        self._add_line('atts = [];')
        self._add_line()

    def add_footer(self):
        super(MatlabNew, self).add_footer()

        self.add_cell('CREATE SCHEMA')        
        self._add_line("schema = struct( ...")
        self._add_line("    'Filename', ncfile, ...")
        self._add_line("    'Format', 'netcdf4', ... choose from {'classic', '64bit', 'netcdf4_classic', 'netcdf4'}")
        self._add_line("    'Name', '/', ...")
        self._add_line("    'Dimensions', dims, ...")
        self._add_line("    'Variables', vars, ...")
        self._add_line("    'Attributes', atts);")

        self._add_line()
        self.add_cell('WRITE SCHEMA')        
        self._add_line('ncwriteschema(ncfile, schema);')

    def add_comment(self, comment):
        self._add_line('%% %s' % comment)

    def add_cell(self, comment):
        self._add_line('%%%% %s' % comment)

    def add_dimension(self, name, length):
        self._add_line('dims(end+1) = struct(\'Name\', \'%s\', \'Value\', %d);' % (name, length))

    def add_variable(self, name, dtype, dimensions):
        self._add_line('var = struct();')
        self._add_line('var.Name = \'%s\';' % name)
        self._add_line('var.Datatype = \'%s\';' % self._convert_dtype(dtype))
        self._add_line('var.Dimensions = {%s};' % ','.join(['\'%s\'' % d for d in dimensions]))
        self._add_line('var.Attributes = [];')

    def add_attribute_global(self, name, value):
        if self._istext(value):
            self._add_line('atts = [atts struct(\'Name\', \'%s\', \'Value\', \'%s\')];' % (name, value))
        else:
            self._add_line('atts = [atts struct(\'Name\', \'%s\', \'Value\', %s)];' % (name, value))

    def add_attribute_var(self, name, value, variable):
        if self._istext(value):
            self._add_line('var.Attributes = [var.Attributes struct(\'Name\', \'%s\', \'Value\', \'%s\')];' % (name, value))
        else:
            self._add_line('var.Attributes = [var.Attributes struct(\'Name\', \'%s\', \'Value\', %s)];' % (name, value))

    def _convert_dtype(self, dtype):
        if dtype.startswith('float'):
            return 'double'
        else:
            return dtype
